package Gtk::Types;
@Gtk::Adjustment::ISA = 'Gtk::Data';
@Gtk::Alignment::ISA = 'Gtk::Bin';
@Gtk::Arrow::ISA = 'Gtk::Misc';
@Gtk::AspectFrame::ISA = 'Gtk::Frame';
@Gtk::Bin::ISA = 'Gtk::Container';
@Gtk::Box::ISA = 'Gtk::Container';
@Gtk::Button::ISA = 'Gtk::Container';
@Gtk::ButtonBox::ISA = 'Gtk::Box';
@Gtk::CheckButton::ISA = 'Gtk::ToggleButton';
@Gtk::CheckMenuItem::ISA = 'Gtk::MenuItem';
@Gtk::ColorSelection::ISA = 'Gtk::VBox';
@Gtk::ColorSelectionDialog::ISA = 'Gtk::Window';
@Gtk::Combo::ISA = 'Gtk::HBox';
@Gtk::Container::ISA = 'Gtk::Widget';
@Gtk::Curve::ISA = 'Gtk::DrawingArea';
@Gtk::Data::ISA = 'Gtk::Object';
@Gtk::Dialog::ISA = 'Gtk::Window';
@Gtk::DrawingArea::ISA = 'Gtk::Widget';
@Gtk::Entry::ISA = 'Gtk::Widget';
@Gtk::EventBox::ISA = 'Gtk::Bin';
@Gtk::FileSelection::ISA = 'Gtk::Window';
@Gtk::Fixed::ISA = 'Gtk::Container';
@Gtk::Frame::ISA = 'Gtk::Bin';
@Gtk::GammaCurve::ISA = 'Gtk::VBox';
@Gtk::HBox::ISA = 'Gtk::Box';
@Gtk::HButtonBox::ISA = 'Gtk::ButtonBox';
@Gtk::HPaned::ISA = 'Gtk::Paned';
@Gtk::HRuler::ISA = 'Gtk::Ruler';
@Gtk::HScale::ISA = 'Gtk::Scale';
@Gtk::HScrollbar::ISA = 'Gtk::Scrollbar';
@Gtk::HSeparator::ISA = 'Gtk::Separator';
@Gtk::HandleBox::ISA = 'Gtk::EventBox';
@Gtk::Image::ISA = 'Gtk::Misc';
@Gtk::InputDialog::ISA = 'Gtk::Window';
@Gtk::Item::ISA = 'Gtk::Bin';
@Gtk::Label::ISA = 'Gtk::Misc';
@Gtk::List::ISA = 'Gtk::Container';
@Gtk::ListItem::ISA = 'Gtk::Item';
@Gtk::Menu::ISA = 'Gtk::MenuShell';
@Gtk::MenuBar::ISA = 'Gtk::MenuShell';
@Gtk::MenuItem::ISA = 'Gtk::Item';
@Gtk::MenuShell::ISA = 'Gtk::Container';
@Gtk::Misc::ISA = 'Gtk::Widget';
@Gtk::Notebook::ISA = 'Gtk::Container';
@Gtk::OptionMenu::ISA = 'Gtk::Button';
@Gtk::Paned::ISA = 'Gtk::Container';
@Gtk::Pixmap::ISA = 'Gtk::Misc';
@Gtk::Preview::ISA = 'Gtk::Widget';
@Gtk::ProgressBar::ISA = 'Gtk::Widget';
@Gtk::RadioButton::ISA = 'Gtk::CheckButton';
@Gtk::RadioMenuItem::ISA = 'Gtk::CheckMenuItem';
@Gtk::Range::ISA = 'Gtk::Widget';
@Gtk::Ruler::ISA = 'Gtk::Widget';
@Gtk::Scale::ISA = 'Gtk::Range';
@Gtk::Scrollbar::ISA = 'Gtk::Range';
@Gtk::ScrolledWindow::ISA = 'Gtk::Container';
@Gtk::Separator::ISA = 'Gtk::Widget';
@Gtk::Table::ISA = 'Gtk::Container';
@Gtk::Text::ISA = 'Gtk::Widget';
@Gtk::ToggleButton::ISA = 'Gtk::Button';
@Gtk::Toolbar::ISA = 'Gtk::Container';
@Gtk::Tree::ISA = 'Gtk::Container';
@Gtk::TreeItem::ISA = 'Gtk::Item';
@Gtk::VBox::ISA = 'Gtk::Box';
@Gtk::VButtonBox::ISA = 'Gtk::ButtonBox';
@Gtk::VPaned::ISA = 'Gtk::Paned';
@Gtk::VRuler::ISA = 'Gtk::Ruler';
@Gtk::VScale::ISA = 'Gtk::Scale';
@Gtk::VScrollbar::ISA = 'Gtk::Scrollbar';
@Gtk::VSeparator::ISA = 'Gtk::Separator';
@Gtk::Viewport::ISA = 'Gtk::Bin';
@Gtk::Widget::ISA = 'Gtk::Object';
@Gtk::Window::ISA = 'Gtk::Bin';
1;
